@@ -23,8 +23,6 @@
 #include <QUuid>
 #include <QString>
 
-#include <kicon.h>
-
 #include "KoDocumentSectionModel.h"
 
 #include "kis_shared.h"
@@ -54,8 +52,6 @@ class KRITAIMAGE_EXPORT KisBaseNode : public QObject, public KisShared
 
 public:
 
-    enum { Visible = 1, Hidden = 2, UserLocked = 4, UserUnlocked = 8, Linked = 16, Unlinked = 32 };
-
     /**
      * Create a new, empty base node. The node is unnamed, unlocked
      * visible and unlinked.
@@ -284,8 +280,11 @@ public:
      *
      * Toggling the visibility of a node will not automatically lead
      * to recomposition.
+     *
+     * @param visible the new visibility state
+     * @param isLoading if true, the property is set during loading.
      */
-    virtual void setVisible(bool v);
+    virtual void setVisible(bool visibile, bool loading = false);
 
     /**
      * Return the locked status of this node. Locked nodes cannot be
@@ -371,6 +370,16 @@ public:
         return QRect();
     }
 
+    /**
+     * Sets the state of the node to the value of @param collapsed
+     */
+    void setCollapsed(bool collapsed);
+
+    /**
+     * returns the collapsed state of this node
+     */
+    bool collapsed() const;
+
 protected:
 
     /**
