@@ -19,7 +19,7 @@
 #ifndef __KIS_IMAGEPIPE_BRUSH_TEST_H
 #define __KIS_IMAGEPIPE_BRUSH_TEST_H
 
-#include <QtTest/QtTest>
+#include <QtTest>
 
 class KisImagePipeBrush;
 
@@ -29,7 +29,11 @@ class KisImagePipeBrushTest : public QObject
 private slots:
     void testLoading();
     void testChangingBrushes();
-    void testDabApplication();
+    void testSimpleDabApplication();
+    void testColoredDab();
+    void testColoredDabWash();
+    void testTextBrushNoPipes();
+    void testTextBrushPiped();
 
 private:
     void checkConsistency(KisImagePipeBrush *brush);
