@@ -1,6 +1,6 @@
 /*
  *  Copyright (c) 2002 Patrick Julien <freak@codepimps.org>
- *  Copyright (c) 2005 Casper Boemann <cbr@boemann.dk>
+ *  Copyright (c) 2005 C. Boemann <cbo@boemann.dk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
 #include <kis_selection_mask.h>
 #include <kis_shape_layer.h>
 #include <kis_transparency_mask.h>
-
+#include <kis_file_layer.h>
 
 using namespace KRA;
 
@@ -54,6 +54,11 @@ KisSaveXmlVisitor::KisSaveXmlVisitor(QDomDocument doc, const QDomElement & eleme
     m_elem = element;
 }
 
+void KisSaveXmlVisitor::setSelectedNodes(vKisNodeSP selectedNodes)
+{
+    m_selectedNodes = selectedNodes;
+}
+
 bool KisSaveXmlVisitor::visit(KisExternalLayer * layer)
 {
     if (layer->inherits("KisShapeLayer")) {
@@ -63,6 +68,17 @@ bool KisSaveXmlVisitor::visit(KisExternalLayer * layer)
         m_count++;
         return saveMasks(layer, layerElement);
     }
+    else if (layer->inherits("KisFileLayer")) {
+        QDomElement layerElement = m_doc.createElement(LAYER);
+        saveLayer(layerElement, FILE_LAYER, layer);
+        layerElement.setAttribute("source", dynamic_cast<KisFileLayer*>(layer)->fileName());
+        layerElement.setAttribute("scale", dynamic_cast<KisFileLayer*>(layer)->scaleToImageResolution() ?  "true" : "false");
+        layerElement.setAttribute(COLORSPACE_NAME, layer->original()->colorSpace()->id());
+
+        m_elem.appendChild(layerElement);
+        m_count++;
+        return saveMasks(layer, layerElement);
+    }
     return false;
 }
 
@@ -103,6 +119,7 @@ bool KisSaveXmlVisitor::visit(KisGroupLayer *layer)
     Q_ASSERT(!layerElement.isNull());
     layerElement.appendChild(elem);
     KisSaveXmlVisitor visitor(m_doc, elem, m_count);
+    visitor.setSelectedNodes(m_selectedNodes);
     m_count++;
     bool success = visitor.visitAllInverse(layer);
 
@@ -134,8 +151,8 @@ bool KisSaveXmlVisitor::visit(KisGeneratorLayer *layer)
 {
     QDomElement layerElement = m_doc.createElement(LAYER);
     saveLayer(layerElement, GENERATOR_LAYER, layer);
-    layerElement.setAttribute(GENERATOR_NAME, layer->generator()->name());
-    layerElement.setAttribute(GENERATOR_VERSION, layer->generator()->version());
+    layerElement.setAttribute(GENERATOR_NAME, layer->filter()->name());
+    layerElement.setAttribute(GENERATOR_VERSION, layer->filter()->version());
     m_elem.appendChild(layerElement);
 
     m_count++;
@@ -207,6 +224,14 @@ void KisSaveXmlVisitor::saveLayer(QDomElement & el, const QString & layerType, c
     el.setAttribute(X, layer->x());
     el.setAttribute(Y, layer->y());
     el.setAttribute(UUID, layer->uuid().toString());
+    el.setAttribute(COLLAPSED, layer->collapsed());
+
+    foreach (KisNodeSP node, m_selectedNodes) {
+        if (node.data() == layer) {
+            el.setAttribute("selected", "true");
+            break;
+        }
+    }
 
     m_nodeFileNames[layer] = LAYER + QString::number(m_count);
 
@@ -227,6 +252,10 @@ void KisSaveXmlVisitor::saveMask(QDomElement & el, const QString & maskType, con
     el.setAttribute(Y, mask->y());
     el.setAttribute(UUID, mask->uuid().toString());
 
+    if (maskType == SELECTION_MASK) {
+        el.setAttribute(ACTIVE, mask->nodeProperties().boolProperty("visible"));
+    }
+
     m_nodeFileNames[mask] = MASK + QString::number(m_count);
 
     dbgFile << "Saved mask "
@@ -242,6 +271,7 @@ bool KisSaveXmlVisitor::saveMasks(KisNode * node, QDomElement & layerElement)
         Q_ASSERT(!layerElement.isNull());
         layerElement.appendChild(elem);
         KisSaveXmlVisitor visitor(m_doc, elem, m_count);
+        visitor.setSelectedNodes(m_selectedNodes);
         bool success =  visitor.visitAllInverse(node);
 
         QMapIterator<const KisNode*, QString> i(visitor.nodeFileNames());
