@@ -31,6 +31,7 @@ class KoCompositeOp;
 class KisPainter;
 class KisPostExecutionUndoAdapter;
 class KisRecordedPaintAction;
+class KisPattern;
 
 
 class KRITAUI_EXPORT KisResourcesSnapshot : public KisShared
@@ -49,6 +50,7 @@ public:
     void setFillStyle(KisPainter::FillStyle fillStyle);
 
     KisNodeSP currentNode() const;
+    KisImageWSP image() const;
     bool needsIndirectPainting() const;
 
     bool needsAirbrushing() const;
@@ -57,6 +59,10 @@ public:
     quint8 opacity() const;
     const KoCompositeOp* compositeOp() const;
 
+    KisPattern* currentPattern() const;
+    KoColor currentFgColor() const;
+    KoColor currentBgColor() const;
+
 private:
     struct Private;
     Private * const m_d;
