@@ -30,6 +30,7 @@
 #include <kaction.h>
 #include <kactioncollection.h>
 
+#include <KoIcon.h>
 #include <KoPointerEvent.h>
 #include <KoViewConverter.h>
 #include <KoCanvasController.h>
@@ -65,14 +66,12 @@
 
 static const int HIDE_OUTLINE_TIMEOUT = 800; // ms
 
-KisToolFreehand::KisToolFreehand(KoCanvasBase * canvas, const QCursor & cursor, const QString & transactionText)
+KisToolFreehand::KisToolFreehand(KoCanvasBase * canvas, const QCursor & cursor, const QString & /*transactionText*/)
     : KisToolPaint(canvas, cursor)
 {
     m_explicitShowOutline = false;
 
-    m_smooth = true;
     m_assistant = false;
-    m_smoothness = 1.0;
     m_magnetism = 1.0;
 
     setSupportOutline(true);
@@ -88,13 +87,13 @@ KisToolFreehand::KisToolFreehand(KoCanvasBase * canvas, const QCursor & cursor,
 
     if (!collection->action("increase_brush_size")) {
         KAction *increaseBrushSize = new KAction(i18n("Increase Brush Size"), collection);
-        increaseBrushSize->setShortcut(Qt::Key_Period);
+        increaseBrushSize->setShortcut(Qt::Key_BracketRight);
         collection->addAction("increase_brush_size", increaseBrushSize);
     }
 
     if (!collection->action("decrease_brush_size")) {
         KAction *decreaseBrushSize = new KAction(i18n("Decrease Brush Size"), collection);
-        decreaseBrushSize->setShortcut(Qt::Key_Comma);
+        decreaseBrushSize->setShortcut(Qt::Key_BracketLeft);
         collection->addAction("decrease_brush_size", decreaseBrushSize);
     }
 
@@ -140,8 +139,8 @@ int KisToolFreehand::flags() const
 void KisToolFreehand::activate(ToolActivation activation, const QSet<KoShape*> &shapes)
 {
     KisToolPaint::activate(activation, shapes);
-    connect(actions().value("increase_brush_size"), SIGNAL(triggered()), SLOT(increaseBrushSize()));
-    connect(actions().value("decrease_brush_size"), SIGNAL(triggered()), SLOT(decreaseBrushSize()));
+    connect(actions().value("increase_brush_size"), SIGNAL(triggered()), SLOT(increaseBrushSize()), Qt::UniqueConnection);
+    connect(actions().value("decrease_brush_size"), SIGNAL(triggered()), SLOT(decreaseBrushSize()), Qt::UniqueConnection);
 }
 
 void KisToolFreehand::deactivate()
@@ -159,7 +158,7 @@ void KisToolFreehand::initStroke(KoPointerEvent *event)
 {
     setCurrentNodeLocked(true);
 
-    m_helper->setSmoothness(m_smooth, m_smoothness);
+    m_helper->setSmoothness(m_smoothingOptions);
     m_helper->initPaint(event, canvas()->resourceManager(),
                         image(),
                         image().data(),
@@ -213,15 +212,26 @@ void KisToolFreehand::mousePressEvent(KoPointerEvent *e)
 
         requestUpdateOutline(e->point);
 
+        if (currentNode() && currentNode()->inherits("KisShapeLayer")) {
+            KisCanvas2 *canvas2 = dynamic_cast<KisCanvas2 *>(canvas());
+            canvas2->view()->showFloatingMessage(i18n("Can't paint on vector layer."), koIcon("draw-brush"));
+        }
+
         if (nodePaintAbility() != PAINT)
             return;
 
+        if (!nodeEditable()) {
+            return;
+        }
+
         setMode(KisTool::PAINT_MODE);
 
         KisCanvas2 *canvas2 = dynamic_cast<KisCanvas2 *>(canvas());
         if (canvas2)
             canvas2->view()->disableControls();
 
+
+        currentPaintOpPreset()->settings()->setCanvasRotation( static_cast<KisCanvas2*>(canvas())->rotationAngle() );
         initStroke(e);
 
         e->accept();
@@ -234,8 +244,6 @@ void KisToolFreehand::mousePressEvent(KoPointerEvent *e)
 
 void KisToolFreehand::mouseMoveEvent(KoPointerEvent *e)
 {
-    requestUpdateOutline(e->point);
-
     /**
      * Update outline
      */
@@ -252,6 +260,7 @@ void KisToolFreehand::mouseMoveEvent(KoPointerEvent *e)
             }
         }
 #endif
+        requestUpdateOutline(e->point);
     }
 
     if (mode() != KisTool::PAINT_MODE) {
@@ -269,7 +278,12 @@ void KisToolFreehand::mouseReleaseEvent(KoPointerEvent* e)
 {
     if (mode() == KisTool::PAINT_MODE &&
             e->button() == Qt::LeftButton) {
-
+        if (m_smoothingOptions.smoothingType == KisSmoothingOptions::WEIGHTED_SMOOTHING) {
+            m_smoothingOptions.smoothingType = KisSmoothingOptions::SIMPLE_SMOOTHING;
+            m_helper->setSmoothness(m_smoothingOptions);
+            doStroke(e);
+            m_smoothingOptions.smoothingType =  KisSmoothingOptions::WEIGHTED_SMOOTHING;
+        }
         endStroke();
 
         if (m_assistant) {
@@ -313,6 +327,11 @@ void KisToolFreehand::keyReleaseEvent(QKeyEvent* event)
     event->accept();
 }
 
+bool KisToolFreehand::isGestureSupported() const
+{
+    return true;
+}
+
 void KisToolFreehand::gesture(const QPointF &offsetInDocPixels, const QPointF &initialDocPoint)
 {
     currentPaintOpPreset()->settings()->changePaintOpSize(offsetInDocPixels.x(), offsetInDocPixels.y());
@@ -324,11 +343,6 @@ bool KisToolFreehand::wantsAutoScroll() const
     return false;
 }
 
-void KisToolFreehand::setSmooth(bool smooth)
-{
-    m_smooth = smooth;
-}
-
 void KisToolFreehand::setAssistant(bool assistant)
 {
     m_assistant = assistant;
@@ -446,6 +460,10 @@ QPainterPath KisToolFreehand::getOutlinePath(const QPointF &documentPos,
         rotation = paintOp->currentRotation();
     }
 
+    if (mode() == KisTool::HOVER_MODE) {
+        rotation += static_cast<KisCanvas2*>(canvas())->rotationAngle() * M_PI / 180.0;
+    }
+
     QPointF imagePos = currentImage()->documentToPixel(documentPos);
     QPainterPath path = currentPaintOpPreset()->settings()->
             brushOutline(imagePos, outlineMode, scale, rotation);
