@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2007, 2009 Thomas Zander <zander@kde.org>
  * Copyright (C) 2006, 2011 Sebastian Sauer <mail@dipe.org>
- * Copyright (C) 2011 Casper Boemann <cbo@kogmbh.com>
+ * Copyright (C) 2011 C. Boemann <cbo@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,19 +37,20 @@ class KoShape;
 class KoStyleManager;
 class KoChangeTracker;
 class QTextLayout;
+class KoTextRangeManager;
 class KoInlineTextObjectManager;
 class KoViewConverter;
 class KoImageCollection;
-class KoTextAnchor;
+class KoShapeAnchor;
 class KoTextLayoutRootArea;
 class KoTextLayoutRootAreaProvider;
 class KoTextLayoutObstruction;
-
+class KoAnchorTextRange;
 
 class TEXTLAYOUT_EXPORT KoInlineObjectExtent
 {
 public:
-    KoInlineObjectExtent(qreal ascent = 0, qreal descent = 0);
+    explicit KoInlineObjectExtent(qreal ascent = 0, qreal descent = 0);
     qreal m_ascent;
     qreal m_descent;
 };
@@ -97,12 +98,19 @@ public:
 
     /// return the currently set manager, or 0 if none is set.
     KoInlineTextObjectManager *inlineTextObjectManager() const;
+    void setInlineTextObjectManager(KoInlineTextObjectManager *manager);
+
+    /// return the currently set manager, or 0 if none is set.
+    KoTextRangeManager *textRangeManager() const;
+    void setTextRangeManager(KoTextRangeManager *manager);
 
     /// return the currently set changeTracker, or 0 if none is set.
     KoChangeTracker *changeTracker() const;
+    void setChangeTracker(KoChangeTracker *tracker);
 
     /// return the currently set styleManager, or 0 if none is set.
     KoStyleManager *styleManager() const;
+    void setStyleManager(KoStyleManager *manager);
 
     /// Returns the bounding rectangle of block.
     QRectF blockBoundingRect(const QTextBlock & block) const;
@@ -130,6 +138,9 @@ public:
     /// are the tabs relative to indent or not
     bool relativeTabs(QTextBlock block) const;
 
+    /// visualize inline objects during paint
+    void showInlineObjectVisualization(bool show);
+
     /// Calc a bounding box rect of the selection
     QRectF selectionBoundingBox(QTextCursor &cursor) const;
 
@@ -142,14 +153,14 @@ public:
     /// reimplemented to always return 1
     virtual int pageCount() const;
 
-    QList<KoTextAnchor *> textAnchors() const;
+    QList<KoShapeAnchor *> textAnchors() const;
 
     /**
      * Register the anchored obstruction  for run around
      *
      * We have the concept of Obstructions which text has to run around in various ways.
      * We maintain two collections of obstructions. The free which are tied to just a position
-     * (tied to pages), and the anchored obstructions which are each anchored to a KoTextAnchor
+     * (tied to pages), and the anchored obstructions which are each anchored to a KoShapeAnchor
      *
      * The free obstructions are collected from the KoTextLayoutRootAreaProvider during layout
      *
@@ -174,6 +185,10 @@ public:
     /// positionInlineObject()
     void setAnchoringParagraphRect(const QRectF &paragraphRect);
 
+    /// Sets the paragraph content rect that will be applied to anchorStrategies being created in
+    /// positionInlineObject()
+    void setAnchoringParagraphContentRect(const QRectF &paragraphContentRect);
+
     /// Sets the layoutEnvironment rect that will be applied to anchorStrategies being created in
     /// positionInlineObject()
     void setAnchoringLayoutEnvironmentRect(const QRectF &layoutEnvironmentRect);
@@ -188,7 +203,7 @@ public:
     void positionAnchoredObstructions();
 
     /// remove inline object
-    void removeInlineObject(KoTextAnchor *textAnchor);
+    void removeInlineObject(KoShapeAnchor *textAnchor);
 
     void clearInlineObjectRegistry(QTextBlock block);
 
@@ -203,6 +218,9 @@ public:
      */
     KoTextLayoutRootArea *rootAreaForPosition(int position) const;
 
+
+    KoTextLayoutRootArea *rootAreaForPoint(const QPointF &point) const;
+
     /**
      * Remove the root-areas \p rootArea from the list of \a rootAreas() .
      * \param rootArea root-area to remove. If NULL then all root-areas are removed.
@@ -212,6 +230,8 @@ public:
     /// reimplemented from QAbstractTextDocumentLayout
     virtual void documentChanged(int position, int charsRemoved, int charsAdded);
 
+    void setContinuationObstruction(KoTextLayoutObstruction *continuationObstruction);
+
     /// Return a list of obstructions intersecting current root area (during layout)
     QList<KoTextLayoutObstruction *> currentObstructions();
 
@@ -232,6 +252,13 @@ public:
     KoTextDocumentLayout* referencedLayout() const;
     void setReferencedLayout(KoTextDocumentLayout *layout);
 
+    /**
+     * To be called during layout by KoTextLayoutArea - similar to how qt calls positionInlineObject
+     *
+     * It searches for anchor text ranges in the given span
+     */
+    void positionAnchorTextRanges(int pos, int length);
+
 signals:
     /**
      * Signal that is emitted during layouting to inform about the progress done so far.
