@@ -19,10 +19,12 @@
 */
 
 #include "KoDockWidgetTitleBar.h"
+#include "KoDockWidgetTitleBar_p.h"
 #include "KoDockWidgetTitleBarButton.h"
 
+#include <KoIcon.h>
+
 #include <kdebug.h>
-#include <kicon.h>
 
 #include <QAbstractButton>
 #include <QAction>
@@ -44,32 +46,6 @@ static inline QDockWidget *parentDock(QWidget *w)
     return qobject_cast<QDockWidget*>(w);
 }
 
-class KoDockWidgetTitleBar::Private
-{
-public:
-    Private(KoDockWidgetTitleBar* thePublic) : thePublic(thePublic),
-            openIcon(thePublic->style()->standardIcon(QStyle::SP_TitleBarShadeButton)),
-            closeIcon(thePublic->style()->standardIcon(QStyle::SP_TitleBarUnshadeButton)),
-            textVisibilityMode(KoDockWidgetTitleBar::FullTextAlwaysVisible)
-    {
-        if (openIcon.isNull())
-            openIcon = KIcon("arrow-down");
-        if (closeIcon.isNull())
-            closeIcon = KIcon("arrow-right");
-    }
-    KoDockWidgetTitleBar* thePublic;
-    KIcon openIcon, closeIcon;
-    QAbstractButton* closeButton;
-    QAbstractButton* floatButton;
-    QAbstractButton* collapseButton;
-
-    KoDockWidgetTitleBar::TextVisibilityMode textVisibilityMode;
-
-    void toggleFloating();
-    void toggleCollapsed();
-    void featuresChanged(QDockWidget::DockWidgetFeatures features);
-};
-
 KoDockWidgetTitleBar::KoDockWidgetTitleBar(QDockWidget* dockWidget)
         : QWidget(dockWidget), d(new Private(this))
 {
@@ -139,13 +115,18 @@ QSize KoDockWidgetTitleBar::sizeHint() const
     }
 
     /*
-     *Calculate the width of title and add to the total width of the docker window when collapsed.
+     * Calculate the width of title and add to the total width of the docker window when collapsed.
      */
     const int titleWidth =
         (d->textVisibilityMode == FullTextAlwaysVisible) ? (q->fontMetrics().width(q->windowTitle()) + 2*mw) :
                                                            0;
 
-    return QSize(buttonWidth /*+ height*/ + 2*mw + 2*fw + titleWidth, height);
+    if (d->preCollapsedWidth > 0) {
+        return QSize(d->preCollapsedWidth, height);
+    }
+    else {
+        return QSize(buttonWidth /*+ height*/ + 2*mw + 2*fw + titleWidth, height);
+    }
 }
 
 void KoDockWidgetTitleBar::paintEvent(QPaintEvent*)
@@ -233,6 +214,8 @@ void KoDockWidgetTitleBar::Private::toggleCollapsed()
     QDockWidget *q = qobject_cast<QDockWidget*>(thePublic->parentWidget());
     if (q == 0) // there does not *have* to be anything on the dockwidget.
         return;
+
+    preCollapsedWidth = q->widget()->isHidden() ? -1 : thePublic->width();
     q->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX); // will be overwritten again next
     if (q->widget()) {
         q->widget()->setVisible(q->widget()->isHidden());
