@@ -40,8 +40,8 @@
 #include <SvgWriter.h>
 #include <SvgStyleWriter.h>
 
-#include <KLocale>
-#include <KDebug>
+#include <klocale.h>
+#include <kdebug.h>
 
 #include <QBuffer>
 #include <QPen>
@@ -65,11 +65,11 @@ ArtisticTextShape::~ArtisticTextShape()
     }
 }
 
-void ArtisticTextShape::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &)
+void ArtisticTextShape::paint(QPainter &painter, const KoViewConverter &converter, KoShapePaintingContext &paintContext)
 {
     applyConversion( painter, converter );
     if( background() )
-        background()->paint( painter, outline() );
+        background()->paint( painter, converter, paintContext, outline() );
 }
 
 void ArtisticTextShape::saveOdf(KoShapeSavingContext &context) const
@@ -1017,7 +1017,7 @@ bool ArtisticTextShape::saveSvg(SvgSavingContext &context)
         context.styleWriter().endElement();
 
         context.shapeWriter().startElement("textPath");
-        context.shapeWriter().addAttribute("xlink:href", "#"+id);
+        context.shapeWriter().addAttribute("xlink:href", QLatin1Char('#')+id);
         if (startOffset() > 0.0)
             context.shapeWriter().addAttribute("startOffset", QString("%1%").arg(startOffset() * 100.0));
         foreach(const ArtisticTextRange &range, formattedText) {
@@ -1053,7 +1053,7 @@ void ArtisticTextShape::saveSvgTextRange(const ArtisticTextRange &range, SvgSavi
         int charIndex = 0;
         while(range.hasXOffset(charIndex)) {
             if (charIndex)
-                attributeValue += ",";
+                attributeValue += QLatin1Char(',');
             attributeValue += QString("%1").arg(SvgUtil::toUserSpace(range.xOffset(charIndex++)));
         }
         context.shapeWriter().addAttribute(attributeName, attributeValue);
@@ -1066,7 +1066,7 @@ void ArtisticTextShape::saveSvgTextRange(const ArtisticTextRange &range, SvgSavi
         int charIndex = 0;
         while(range.hasYOffset(charIndex)) {
             if (charIndex)
-                attributeValue += ",";
+                attributeValue += QLatin1Char(',');
             attributeValue += QString("%1").arg(SvgUtil::toUserSpace(baselineOffset+range.yOffset(charIndex++)));
         }
         context.shapeWriter().addAttribute(attributeName, attributeValue);
@@ -1076,7 +1076,7 @@ void ArtisticTextShape::saveSvgTextRange(const ArtisticTextRange &range, SvgSavi
         int charIndex = 0;
         while(range.hasRotation(charIndex)) {
             if (charIndex)
-                attributeValue += ",";
+                attributeValue += ',';
             attributeValue += QString("%1").arg(range.rotation(charIndex++));
         }
         context.shapeWriter().addAttribute("rotate", attributeValue);
